var game = new Phaser.Game(800, 600, Phaser.AUTO, '', { preload: function() { this.state.add('GamePlay', GamePlay, true); } });
var logic;
var cursors;

const PLAYER_SPEED	= 200;
const WAIT_MENUSTEP	= 100;
const WAIT_TALK		= 250;

const MENUITEM_TALK		=	0;
const MENUITEM_LEAVE	=	1;
const MENUITEM_TAKE		=	2;

const GUI_MENUITEM_DISTANCE		=	20;

window.addEventListener("keydown", function(e) {
	// Prevent default browser action for arrows and spacebar
	if([32, 37, 38, 39, 40].indexOf(e.keyCode) > -1) {
	e.preventDefault();
	}
}, false);

function hasTimePassed(time, delay) {
	return (game.time.now - time) > delay;
}

class Dialogue {
	// dialogue is array of {actor, text} records.
	constructor(dialogue) {
		this.dialogue = dialogue;
		this.state = 0;
	}

	actual() {
		return this.dialogue[this.state];
	}

	advance() {
		this.state++;
	}
}

class Door extends Phaser.TileSprite {
	constructor(x, y, name, rotation, vector, longpanel) {
		super(game, x, y, 64, 64, 'objects');
		this.name = name;
		this.longpanel = longpanel;
		if (!longpanel) {
			this.anchor = new Phaser.Point(0.5, 0.5);
			this.tilePosition = new Phaser.Point(-64, -64);
			this.openvector = Phaser.Point.multiply(vector, new Phaser.Point(56, 56));
		}
		else {
			this.width *= 2;
			this.anchor = new Phaser.Point(0.75, 0.5);
			this.tilePosition = new Phaser.Point(0, -64);
			this.openvector = Phaser.Point.multiply(vector, new Phaser.Point(116, 116));
		}
		if (rotation === undefined) rotation = 0;
		this.rotation = rotation * (Math.PI / 180);
		this.closetween = game.add.tween(this).to({ x: this.position.x, y: this.position.y }, 1000, Phaser.Easing.Sinusoidal.InOut, false, 0, 0, false);
		this.openposition = Phaser.Point.add(this.position, this.openvector);
		this.opentween = game.add.tween(this).to({ x: this.openposition.x, y: this.openposition.y }, 1000, Phaser.Easing.Sinusoidal.InOut, false, 0, 0, false);
		this.isOpen = false;
		game.physics.arcade.enable(this);
		this.body.immovable = true;
		this.setBody(rotation);
	}

	setBody(rotation) {
		switch(rotation) {
			case 0:
			case 180:
				this.body.setSize(this.width, 10, this.longpanel * (rotation / 180) * 64, 27);
				break;
			case 90:
			case 270:
				this.body.setSize(10, this.width, 27 + (this.longpanel * 64), this.longpanel * ((rotation - 270) / 180) * 64);
				break;
			default:
				console.log("Unable to set body due to unknown rotation:", rotation);
		}
	}

	open() {
		if (!this.isOpen) {
			this.opentween.start();
			this.isOpen = true;
		}
	}

	close() {
		if (this.isOpen) {
			this.closetween.start();
			this.isOpen = false;
		}
	}

	update() {
		game.debug.body(this);
	}
}

class Player extends Phaser.Sprite {
	constructor(x, y) {
		super(game, x, y, 'player');
		this.y -= this.height;
		this.shortname = "John";
		this.fullname = "John Evals";
		this.offerInteraction(null);
		this.unfreeze();
	}

	enablePhysics() {
		game.physics.arcade.enable(this);
		this.body.bounce.y = 0.2;
		this.body.bounce.x = 0.2;
		this.body.collideWorldBounds = true;
	}

	control() {
		if ((this.alive) && (!this.freezed))
		{
			if (cursors.left.isDown)
			{
				this.body.velocity.x = -PLAYER_SPEED;
			}
			else if (cursors.right.isDown)
			{
				this.body.velocity.x = PLAYER_SPEED;
			}
			if (cursors.up.isDown)
			{
				this.body.velocity.y = -PLAYER_SPEED;
			}
			else if (cursors.down.isDown)
			{
				this.body.velocity.y = PLAYER_SPEED;
			}
		}
	}

	update() {
		this.body.velocity.x = this.body.velocity.y = 0;
		this.control();
	}

	freeze() {
		this.freezed = true;
	}

	unfreeze() {
		this.freezed = false;
	}

	offerInteraction(npc) {
		this.interactablenpc = npc;
	}

	takeMe(npc) {
		this.loadTexture(npc.key);
		npc.kill();
		this.disguise = npc;
	}
}

class GameNPC extends Phaser.Sprite {
	constructor(x, y, key, shortname, fullname, interaction_distance) {
		super(game, x, y, key);
		this.y -= this.height;
		this.shortname = shortname;
		this.fullname = fullname;
		this.interaction_distance = interaction_distance;
		this.interactable = false;
		this.talkcount = 0;
	}

	kill() {
		super.kill();
		this.exists = false;
	}

	update() {
		super.update();
		if ((!this.interactable) && (game.physics.arcade.distanceBetween(this, logic.player) < this.interaction_distance)) {
			logic.gameinterface.dropNotice("(ENTER) Interact with " + this.shortname + "!");
			logic.player.offerInteraction(this);
			this.interactable = true;
		}
		else if ((this.interactable) && (game.physics.arcade.distanceBetween(this, logic.player) > this.interaction_distance)) {
			logic.gameinterface.clearNotice();
			logic.player.offerInteraction(null);
			this.interactable = false;
		}
		if ((this.teleport_instructions) && (game.physics.arcade.distanceBetween(this, logic.player) > (game.width + 200))) {
			this.position = this.teleport_instructions.newPosition;
			this.teleport_instructions.callback(this.teleport_instructions.callbackValue);
			this.teleport_instructions = null;
		}
	}

	actionTalk() {
		this.talkcount++;
	}

	actionLeave() {
	}

	actionTake() {
		logic.player.offerInteraction(null);
		logic.player.takeMe(this);
		return true;
	}

	endTalk() {
		return true;
	}

	offscreenTeleportation(newPosition, callback, callbackValue) {
		this.teleport_instructions = { newPosition: newPosition, callback: callback, callbackValue: callbackValue };
		console.log("Offscreen teleport request registered:", this.teleport_instructions);
	}
}

class NPC_Clara extends GameNPC {
	actionTalk() {
		switch (this.talkcount) {
			case 0:
				logic.gameinterface.talk(new Dialogue( [ { actor: this, text: "What a morning..." },
					{ actor: logic.player, text: "Hi Clara! What happened?" },
					{ actor: this, text: "No one cares to tell. I just know that I have to log on people manually, as the access control system doesn't work properly." },
					{ actor: logic.player, text: "Hmm... I'll look into it. Maybe it's just that everyone is fired." },
					{ actor: this, text: "Haha! Wouldn't joke about this, though, due to the recent layoffs." },
					{ actor: logic.player, text: "Well, maybe if we dare to joke about it, it won't happen to us..." },
					{ actor: this, text: "Wish it worked like that... Is it some superstition like the belief that having an umbrella with you prevents rain?" },
					{ actor: logic.player, text: "Nah, that actually works; it's not a superstition, but Murphy's Law!" },
					{ actor: this, text: "If you say so..." },
					{ actor: this, text: "Anyway, I logged you in and opened the door for you." } ] ));
				logic.openDoor("cutedoor");
				break;
			case 1:
				logic.gameinterface.talk(new Dialogue( [ { actor: this, text: "John, have you ever thought about losing your employee card?" },
					{ actor: logic.player, text: "Yeah, you'd just get a new one." },
					{ actor: this, text: "You don't understand me, John!" },
					{ actor: this, text: "..." },
					{ actor: this, text: "I mean... Losing it for real..." },
					{ actor: this, text: "Doesn't it feel like it's the culmination of your being?" },
					{ actor: this, text: "If I had no card, would I still exist?" },
					{ actor: logic.player, text: "..." },
					{ actor: logic.player, text: "You sound very philosophical today." } ] ));
				break;
			case 2:
			case 3:
			case 4:
				logic.gameinterface.talk(new Dialogue( [ { actor: this, text: "Would you like to hear a random fun fact?" },
					{ actor: logic.player, text: "Of course!" },
					{ actor: this, text: "Lorem Ipsum is simply dummy text of the printing and typesetting industry. Lorem Ipsum has been the industry's standard dummy text ever since the 1500s," },
					{ actor: this, text: "when an unknown printer took a galley of type and scrambled it to make a type specimen book. It has survived not only five centuries, but also the leap" },
					{ actor: this, text: "into electronic typesetting, remaining essentially unchanged. It was popularised in the 1960s with the release of Letraset sheets containing Lorem" },
					{ actor: this, text: "Ipsum passages, and more recently with desktop publishing software like Aldus PageMaker including versions of Lorem Ipsum." },
					{ actor: logic.player, text: "That's very interesting, I didn't know that!" },
					{ actor: logic.player, text: "Thanks for sharing, Clara!" } ] ));
				break;
			case 5:
				logic.gameinterface.talk(new Dialogue( [ { actor: this, text: "Would you like to hear a random fun fact?" },
					{ actor: logic.player, text: "Of course!..." },
					{ actor: logic.player, text: "But first..." },
					{ actor: this, text: "What is it?" },
					{ actor: logic.player, text: "..." },
					{ actor: logic.player, text: "Never mind, go on with what you wanted to say." },
					{ actor: this, text: "Lorem Ipsum is simply dummy text of the printing and typesetting industry..." },
					{ actor: logic.player, text: "I knew that already..." },
					{ actor: logic.player, text: "But..." },
					{ actor: logic.player, text: "Would you go out on a date with me?" },
					{ actor: this, text: "..." },
					{ actor: this, text: "No." } ] ));
				break;
			default:
				logic.gameinterface.talk(new Dialogue( [ { actor: this, text: "..." } ] ));
		}
		super.actionTalk();
	}

	actionLeave() {
		logic.gameinterface.dropNotice(this.shortname + ": Have a great day, John!");
	}

	actionTake() {
		logic.openDoor("cutedoor");
		return super.actionTake();
	}

	endTalk() {
		return (this.talkcount < 6);
	}
}

class NPC_Carlos extends GameNPC {
	actionTalk() {
		if (logic.clara.alive) {
			switch (this.talkcount) {
				case 0:
					logic.gameinterface.talk(new Dialogue( [ { actor: logic.player, text: "Good morning, Carlos! What's up?" },
						{ actor: this, text: "Hey, John! Not too much, I'm just trying to sell some shit to this new client." },
						{ actor: logic.player, text: "And so? Is it hard to convince them?" },
						{ actor: this, text: "Of course, they don't want to buy. It's shit." },
						{ actor: this, text: "I mean, who would want to buy shit?" },
						{ actor: logic.player, text: "Yeah. Relatable... But why are you trying to sell shit? I mean, why not one of our cutting edge technologies?" },
						{ actor: this, text: "They couldn't afford it." },
						{ actor: logic.player, text: "Then why are you trying to sell them... anything at all? They'd better off with a cheaper company." },
						{ actor: this, text: "Company policies. Plus the management instructed me to sell to them, no matter what." },
						{ actor: logic.player, text: "That sucks... Well... good luck!" },
						{ actor: this, text: "Thanks..." } ] ));
					logic.closeDoor("cutedoor");
					logic.openDoor("carlosdoor");
					break;
				case 1:
					logic.gameinterface.talk(new Dialogue( [ { actor: this, text: "*Sigh.* I'll probably not work here for long, if I can't sell this shit to the customer." },
						{ actor: this, text: "These layoffs are terrifying. They let go of anyone for even the slightest mistake." },
						{ actor: logic.player, text: "But it's not a mistake when our client doesn't buy something they don't need." },
						{ actor: this, text: "Tell this to my boss..." } ] ));
					break;
				case 2:
					logic.gameinterface.talk(new Dialogue( [ { actor: this, text: "You know, generally, I think it sucks being a child. One thing I miss though, if you're a child, you're allowed to be silly." },
						{ actor: this, text: "..." },
						{ actor: this, text: "For example, when my Dad told me that semen is the seed of animals, I theoretized, if I'd bury some pig semen in the garden, a pig would grow out of it..." },
						{ actor: this, text: "I was totally fascinated by the thought, because I really wanted to have a pet pig, but Dad wouldn't let me have one." },
						{ actor: this, text: "So I thought I could grow my own pig in the garden from pig seeds!" },
						{ actor: logic.player, text: "Interesting. Where are you going with this?" },
						{ actor: this, text: "I don't know... sometimes I feel like I'd rather sell pig semen to our clients, heh!" },
						{ actor: this, text: "Uhm..." },
						{ actor: this, text: "Totally don't tell this to Saiki!" } ] ));
					this.pigsemen = true;
					break;
				case 3:
					logic.gameinterface.talk(new Dialogue( [ { actor: this, text: "Weren't you heading to the IT department to the right?" },
						{ actor: this, text: "You know... the green department which looks way better than blue!" },
						{ actor: this, text: "..." },
						{ actor: this, text: "Damn, why do I have to work in blue? I just feel... so blue about it." } ] ));
					break;
				case 4:
					logic.gameinterface.talk(new Dialogue( [ { actor: this, text: "Weren't you heading to the conveniently green department to the right? I guess Saiki is waiting for you." },
						{ actor: this, text: "You're so lucky that you can work with her every day!" } ] ));
					break;
				default:
					logic.gameinterface.talk(new Dialogue( [ { actor: this, text: "Guess you'll stay here then. It's cool, you're great company anyway." } ] ));
			}
		}
		else {
			switch (this.talkcount) {
				case 0:
					logic.gameinterface.talk(new Dialogue( [ { actor: logic.player, text: "Good morning, Carlos!" },
						{ actor: this, text: "Whoa, Clara! You have changed so much! I barely recognize you. What's up?" },
						{ actor: logic.player, text: "I need to talk to Saiki at the IT department, but... my card doesn't allow me to enter there." },
						{ actor: this, text: "Well... I can certainly help you with that..." },
						{ actor: this, text: "..." } ] ));
					break;
				case 1:
					logic.gameinterface.talk(new Dialogue( [ { actor: logic.player, text: "Carlos. You promised to help me." },
						{ actor: this, text: "Uhm... It's just... Please give me 5 mins to finish this e-mail." },
						{ actor: logic.player, text: "I don't have much time, Carlos." },
						{ actor: this, text: "See, it's just 5 mins, please be patient. I have a chocolate bar in the fridge. I give it to you. Feel free to have it meanwhile." },
						{ actor: logic.player, text: "..." } ] ));
					break;
				case 2:
					logic.gameinterface.talk(new Dialogue( [ { actor: logic.player, text: "Time is up, Carlos!" },
						{ actor: this, text: "Please give me just another min." },
						{ actor: logic.player, text: "Fine. Please just give me your card and I'll help myself!" },
						{ actor: this, text: "But... you know that company policies forbid that, Clara! Please just be a little more patient!" },
						{ actor: logic.player, text: "You will give me your card now or you will regret!" },
						{ actor: this, text: "...?" },
						{ actor: this, text: "You're not gonna treat me like this. Forget it!" },
						{ actor: logic.player, text: "You'll not work here tomorrow." } ] ));
					break;
				default:
					logic.gameinterface.talk(new Dialogue( [ { actor: this, text: "..." } ] ));
			}
		}
		super.actionTalk();
	}

	actionLeave() {
		if (logic.clara.alive) {
			logic.gameinterface.dropNotice(this.shortname + ": Bye, John! Pass on my greetings to Saiki!");
		}
	}

	actionTake() {
		logic.closeDoor("cutedoor");
		logic.openDoor("carlosdoor");
		return super.actionTake();
	}

	endTalk() {
		return (logic.clara.alive) || (this.talkcount < 3);
	}
}

class NPC_Saiki extends GameNPC {
	actionTalk() {
		if (logic.carlos.alive) {
			switch (this.talkcount) {
				case 0:
					logic.gameinterface.talk(new Dialogue( [ { actor: logic.player, text: "Hi Saiki!" },
						{ actor: this, text: "Hi John! Glad you arrived, here's some work for you!" },
						{ actor: logic.player, text: "Awesome! What is it?" },
						{ actor: this, text: "Haha, joking! Actually, our queue is suspiciously empty for today." },
						{ actor: logic.player, text: "Yeah... That's really suspicious!" },
						{ actor: logic.player, text: "Maybe I can get home today in time?" },
						{ actor: this, text: "You mean, you wouldn't spend the mandatory free extra working hours?" },
						{ actor: this, text: "You steal from the company! The company needs your unpaid hours of work! Otherwise, how would we finance the trash can I ordered for the department?" },
						{ actor: this, text: "*Whispers with wink.* I hope you get my sarcasm." },
						{ actor: logic.player, text: "Haha! I always get it, Saiki!" } ] ));
					logic.closeDoor("carlosdoor");
					logic.openDoor("saikidoor");
					break;
				case 1:
					logic.gameinterface.talk(new Dialogue( [ { actor: this, text: "Now that I think about it, there's still one thing. You could look into why the access control system is acting so funny today." },
						{ actor: this, text: "Clara has to enter people manually." },
						{ actor: logic.player, text: "Yeah, I'll definitely check that." } ] ));
					break;
				case 2:
					logic.gameinterface.talk(new Dialogue( [ { actor: logic.player, text: "So... what's up with the trash can?" },
						{ actor: this, text: "I'm always up for such deep conversation" },
						{ actor: this, text: "about trash cans." },
						{ actor: this, text: "While we only have one trash can, other departments seem to be abundant of them. And ours is always filled. So I requested an additional trash can." },
						{ actor: this, text: "Now the ticket is in PENDING status. The latest work note is from two months ago, telling there is no budget at the moment." },
						{ actor: logic.player, text: "Hmm... Maybe it's too much to ask. Can we take one from another department?" },
						{ actor: this, text: "No, everyone holds on to their trash cans." },
						{ actor: logic.player, text: "You certainly did your research." },
						{ actor: this, text: "Of course. It's an important subject." } ] ));
					break;
				case 3:
					logic.gameinterface.talk(new Dialogue( [ { actor: this, text: "My parents named me after a Japanese rock singer." },
						{ actor: logic.player, text: "Cool! Which band?" },
						{ actor: this, text: "I don't remember the band name. But the particular singer has a blue rose in her hair most of the time." } ] ));
					break;
				default:
					if ((logic.carlos.pigsemen) && (this.talkcount == 4)) {
						logic.gameinterface.talk(new Dialogue( [ { actor: logic.player, text: "Did you know that Carlos thought he can grow a pig by burying pig semen in the ground? What do you think about this?" },
							{ actor: this, text: "Hmm..." },
							{ actor: this, text: "I guess it didn't work." },
							{ actor: this, text: "So do you want to know what I think about this?" },
							{ actor: this, text: "..." },
							{ actor: this, text: "IT'S BRILLIANT!!!" },
							{ actor: this, text: "You know, it really sparkled my artistic vision! Now I must draw a pig with its legs rooted into the ground!" },
							{ actor: logic.player, text: "That's gross. Poor pig." },
							{ actor: this, text: "Indeed. But this is the point. He's stuck in one place, cannot move. Like you in life. So you feel sympathy for him." },
							{ actor: logic.player, text: "You have a point." },
							{ actor: this, text: "I think I'll catch Carlos and talk to him. I always liked that guy but I didn't know he's such imaginative." } ] ));
						this.offscreenTeleportation(new Phaser.Point(logic.carlos.x, logic.carlos.y + 64), this.tpDone, this);
					}
					else {
						if (Phaser.Math.random() < 0.5) {
							logic.gameinterface.talk(new Dialogue( [ { actor: this, text: "I've found an orchestrated rendition of „Potential for Anything”. You'll have to check it out, it's breathtaking!" } ] ));
						}
						else {
							logic.gameinterface.talk(new Dialogue( [ { actor: this, text: "I don't like that our department is green. The blue one, where Carlos works, looks much better." } ] ));
						}
					}
			}
		}
		else {
			switch (this.talkcount) {
				case 0:
					logic.gameinterface.talk(new Dialogue( [ { actor: logic.player, text: "I love you, Saiki!" },
						{ actor: this, text: "Hi Carlos..." },
						{ actor: logic.player, text: "I'm infatuated by the mere thought of you." },
						{ actor: this, text: "Well... it sounds pretty much creepy." },
						{ actor: logic.player, text: "Will you go out on a date with me?" },
						{ actor: this, text: "..." },
						{ actor: this, text: "Sorry to hurt your feelings, but... even though I like you... Your sudden, unprecedented confession makes me scared." },
						{ actor: this, text: "Especially with that facial expression." },
						{ actor: this, text: "I mean... you're not even joking." },
						{ actor: this, text: "So sorry... I won't date you." } ] ));
					break;
				case 1:
					logic.gameinterface.talk(new Dialogue( [ { actor: logic.player, text: "I thought I could impress you." },
						{ actor: this, text: "I'm sorry." } ] ));
					break;
				case 2:
					logic.gameinterface.talk(new Dialogue( [ { actor: logic.player, text: "You will give me your card." },
						{ actor: this, text: "Geez, what for? Go back to your department and work!" },
						{ actor: logic.player, text: "I need it." },
						{ actor: this, text: "..." } ] ));
					break;
				default:
					logic.gameinterface.talk(new Dialogue( [ { actor: this, text: "Don't you have anything better to do, Carlos?" } ] ));
			}
		}
		super.actionTalk();
	}

	actionLeave() {
		if (logic.carlos.alive) {
			logic.gameinterface.dropNotice(this.shortname + ": It was nice talking to you!");
		}
	}

	actionTake() {
		logic.closeDoor("carlosdoor");
		logic.openDoor("saikidoor");
		return super.actionTake();
	}

	endTalk() {
		return (logic.carlos.alive);
	}

	tpDone(value) {
		console.log("Woot-woot! Teleport complete:", value);
	}
}


class GameInterface extends Phaser.Group {
	constructor(game, parent) {
		super(game, parent, 'GUI', false, false, 0);
		this.back_notice = new Phaser.Graphics(game, 0, game.height - 50);
		this.back_notice.beginFill(0x000000);
		this.back_notice.drawRect(0, 0, game.width, 30);
		this.back_notice.endFill();
		this.add(this.back_notice);
		this.text_notice = new Phaser.Text(game, 0, this.back_notice.y, null, { align: 'left', fill: 'white', font: 'Ubuntu Mono', fontSize: 18, fontWeight: 'bold' });
		this.text_notice.anchor.setTo(-0.5, -0.2);
		this.add(this.text_notice);
		this.clearNotice();

		this.back_menu = new Phaser.Graphics(game, 150, (game.height / 2) + 40);
		this.back_menu.beginFill(0x000000);
		this.back_menu.drawRect(0, 0, 155, 105);
		this.back_menu.endFill();
		this.back_menu.lineStyle(3, Phaser.Color.YELLOW, 1);
		this.back_menu.moveTo(10, 35);
		this.back_menu.lineTo(this.back_menu.width - 10, 35);
		this.add(this.back_menu);
		var style = { align: 'left', fill: 'yellow', font: 'Ubuntu Mono', fontSize: 22, fontWeight: 'bold' };
		this.text_menutitle = new Phaser.Text(game, this.back_menu.x, this.back_menu.y, null, style);
		this.text_menutitle.anchor.setTo(-0.2, -0.2);
		style = { align: 'left', fill: 'white', font: 'Ubuntu Mono', fontSize: 18, fontWeight: 'bold' };
		this.text_menuitem_Talk		=	new Phaser.Text(game, this.back_menu.x + 35, this.back_menu.y + 40, "Talk", style);
		this.text_menuitem_Leave	=	new Phaser.Text(game, this.back_menu.x + 35, this.text_menuitem_Talk.y + GUI_MENUITEM_DISTANCE, "Leave", style);
		this.text_menuitem_Take		=	new Phaser.Text(game, this.back_menu.x + 35, this.text_menuitem_Leave.y + GUI_MENUITEM_DISTANCE, "Take ID", style);
		style.fill = 'yellow';
		this.text_menucursor		=	new Phaser.Text(game, this.back_menu.x + 15, this.text_menuitem_Talk.y, "→", style);
		this.add(this.text_menutitle);
		this.add(this.text_menuitem_Talk);
		this.add(this.text_menuitem_Leave);
		this.add(this.text_menuitem_Take);
		this.add(this.text_menucursor);
		this.leaveMenu();
		this.last_menustep = 0;

		this.back_talk = new Phaser.Graphics(game, 100, game.height - 120);
		this.back_talk.beginFill(0x000000);
		this.back_talk.drawRect(0, 0, game.width - 200, 105);
		this.back_talk.endFill();
		this.back_talk.lineStyle(3, Phaser.Color.YELLOW, 1);
		this.back_talk.moveTo(10, 35);
		this.back_talk.lineTo(this.back_talk.width - 10, 35);
		this.add(this.back_talk);
		style = { align: 'left', fill: 'yellow', font: 'Ubuntu Mono', fontSize: 22, fontWeight: 'bold' };
		this.text_talktitle = new Phaser.Text(game, this.back_talk.x + 15, this.back_talk.y + 6, null, style);
		//this.text_talktitle.anchor.setTo(-0.2, -0.2);
		this.add(this.text_talktitle);
		this.strikethrough_talktitle = new Phaser.Graphics(game, this.back_talk.x, this.back_talk.y);
		this.strikethrough_talktitle.lineStyle(3, Phaser.Color.RED, 1);
		this.strikethrough_talktitle.moveTo(12, 20);
		this.strikethrough_talktitle.lineTo(67, 20);
		this.add(this.strikethrough_talktitle);
		style = { align: 'left', fill: 'white', font: 'Ubuntu Mono', fontSize: 18, fontWeight: 'bold' };
		this.text_talk = new Phaser.Text(game, this.back_talk.x + 35, this.back_talk.y + 40, null, style);
		this.text_talk.wordWrap = true;
		this.text_talk.wordWrapWidth = this.back_talk.width - 70;
		this.text_talk.lineSpacing = -5;
		this.add(this.text_talk);
		this.leaveTalk();
		this.last_talk = 0;
	}

	dropNotice(text) {
		this.text_notice.text = text;
		this.back_notice.visible = true;
		this.text_notice.visible = true;
	}

	clearNotice() {
		this.back_notice.visible = false;
		this.text_notice.visible = false;
	}

	npcMenu(npc) {
		if (!this.inMenu) {
			this.inMenu = true;
			this.clearNotice();
			this.text_menutitle.text = npc.shortname;
			this.back_menu.visible = true;
			this.text_menutitle.visible = true;
			this.text_menuitem_Talk.visible = true;
			this.text_menuitem_Leave.visible = true;
			this.text_menuitem_Take.visible = true;
			this.currentmenuitem = MENUITEM_TALK;
			this.actualizeCursorPosition();
			this.text_menucursor.visible = true;
			this.last_menustep = game.time.now;
		}
	}

	leaveMenu() {
		this.back_menu.visible = false;
		this.text_menutitle.visible = false;
		this.text_menuitem_Talk.visible = false;
		this.text_menuitem_Leave.visible = false;
		this.text_menuitem_Take.visible = false;
		this.text_menucursor.visible = false;
		this.inMenu = false;
	}

	talk(dialogue) {
		this.inTalk = true;
		this.dialogue = dialogue;
		this.advanceTalk();
		this.back_talk.visible = true;
		this.text_talktitle.visible = true;
		this.text_talk.visible = true;
	}

	leaveTalk() {
		this.inTalk = false;
		this.dialogue = null;
		this.back_talk.visible = false;
		this.text_talktitle.visible = false;
		this.strikethrough_talktitle.visible = false;
		this.text_talk.visible = false;
	}

	advanceTalk() {
		console.log(this.dialogue);
		console.log(this.dialogue.actual());
		var actualdialogue = this.dialogue.actual();
		if (actualdialogue) {
			if (actualdialogue.actor.disguise) {
				this.text_talktitle.text = actualdialogue.actor.shortname + " " + actualdialogue.actor.disguise.shortname;
				this.strikethrough_talktitle.visible = true;
			}
			else {
				this.text_talktitle.text = actualdialogue.actor.shortname;
				this.strikethrough_talktitle.visible = false;
			}
			this.text_talk.text = actualdialogue.text;
			this.dialogue.advance();
		}
		else {
			this.leaveTalk();
			logic.endTalk();
		}
		this.last_talk = game.time.now;
	}

	actualizeCursorPosition() {
		this.text_menucursor.y = this.text_menuitem_Talk.y + (this.currentmenuitem * GUI_MENUITEM_DISTANCE);
	}

	update() {
		if ((this.inMenu) && (hasTimePassed(this.last_menustep, WAIT_MENUSTEP))) {
			if (cursors.up.isDown)
			{
				this.currentmenuitem--;
				if (this.currentmenuitem < MENUITEM_TALK) this.currentmenuitem = MENUITEM_TAKE;
				this.actualizeCursorPosition();
			}
			else if (cursors.down.isDown)
			{
				this.currentmenuitem++;
				if (this.currentmenuitem > MENUITEM_TAKE) this.currentmenuitem = MENUITEM_TALK;
				this.actualizeCursorPosition();
			}
			if (game.input.keyboard.isDown(Phaser.Keyboard.ENTER)) {
				logic.callMenu(this.currentmenuitem);
			}
			this.last_menustep = game.time.now;
		}
		if ((this.inTalk) && (game.input.keyboard.isDown(Phaser.Keyboard.ENTER)) && (hasTimePassed(this.last_talk, WAIT_TALK))) {
			this.advanceTalk();
		}
	}
}

class GameLogic {

	constructor() {
		this.player = this.clara = this.carlos = this.saiki = this.peter = this.bianca = null;
		this.gameinterface = new GameInterface(game, game.stage);
		//this.doors = game.add.group(game.world, "doors");
		this.last_menuselect = 0;
	}

	createObject(object) {
		switch (object.type) {
			case 'spawnpoint':		this.createCharacter(object); break;
			case 'door':			this.createDoor(object); break;
			case '':				console.error("Object type is empty:", object); break;
			default:				console.error("Unknown object type:", object);
		}
	}

	createCharacter(object) {
		var newChar;
		switch (object.name) {
			case 'john':
				newChar = new Player(object.x, object.y, 'player'); 
				this.player = newChar;
				this.player.enablePhysics();
				game.camera.follow(this.player);
				break;
			case 'clara':
				newChar = new NPC_Clara(object.x, object.y, 'clara', "Clara", "Clara Tnavelerri", 200);
				this.clara = newChar;
				break;
			case 'carlos':
				newChar = new NPC_Carlos(object.x, object.y, 'carlos', "Carlos", "Carlos Elbacalper", 150);
				this.carlos = newChar;
				break;
			case 'saiki':
				newChar = new NPC_Saiki(object.x, object.y, 'saiki', "Saiki", "Saiki Ytpme", 150);
				this.saiki = newChar;
				break;
			default:
				console.error("Unknown character:", object);
		}
		newChar.name = object.name;
		game.add.existing(newChar);
		console.log(newChar);
	}

	createDoor(object) {
		/* Calculate movement vector and correct position (32 = half tile width/height). */
		var vector;
		switch (object.rotation) {
			case undefined:
			case 0:		vector = new Phaser.Point( -1,  0); object.x += 32; object.y -= 32; break;
			case 90:	vector = new Phaser.Point(  0, -1); object.x += 32; object.y += 32; break;
			case 180:	vector = new Phaser.Point(  1,  0); object.x -= 32; object.y += 32; break;
			case 270:	vector = new Phaser.Point(  0,  1); object.x -= 32; object.y -= 32; break;
			default:	console.error("Invalid rotation:", object.rotation);
		}
		this.doors.add(new Door(object.x, object.y, object.name, object.rotation, vector, object.properties.longpanel));
	}

	callMenu(menuitem) {
		console.log("Menu callback received:", menuitem);
		this.last_menuselect = game.time.now;
		this.gameinterface.leaveMenu();
		switch (menuitem) {
			case MENUITEM_TALK:
				this.player.interactablenpc.actionTalk();
				break;
			case MENUITEM_LEAVE:
				this.player.interactablenpc.actionLeave();
				this.player.unfreeze();
				break;
			case MENUITEM_TAKE:
				if (this.player.interactablenpc.actionTake())
					this.player.unfreeze();
				break;
		}
	}

	endTalk() {
		if (this.player.interactablenpc) {
			if (this.player.interactablenpc.endTalk()) {
				this.gameinterface.npcMenu(this.player.interactablenpc);
			}
			else {
				this.player.unfreeze();
				this.last_menuselect = game.time.now;
			}
		}
	}

	openDoor(doorname) {
		this.doors.children.forEach(function(o) { if (o.name == doorname) o.open(); });
	}

	closeDoor(doorname) {
		this.doors.children.forEach(function(o) { if (o.name == doorname) o.close(); });
	}

	update() {
		if ((game.input.keyboard.isDown(Phaser.Keyboard.ENTER)) && (hasTimePassed(this.last_menuselect, WAIT_MENUSTEP))) {
			if ((this.player.interactablenpc) && (this.player.interactablenpc.interactable) && (!this.gameinterface.inMenu) && (!this.gameinterface.inTalk)) {
				console.log("Starting interaction with", this.player.interactablenpc.fullname);
				this.player.freeze();
				this.gameinterface.npcMenu(this.player.interactablenpc);
			}
		}
	}

}

class GamePlay extends Phaser.State {

	constructor() {

		super();
		logic = new GameLogic();
		game.world.updateOnlyExistingChildren = true;
		//game.state.add('GameOver', GameOver, false);

	}

	preload() {

		game.load.image('player', 'john.png');
		game.load.image('clara', 'clara.png');
		game.load.image('carlos', 'carlos.png');
		game.load.image('saiki', 'saiki.png');
		game.load.image('tileset', 'tileset.png');
		game.load.image('objects', 'objects.png');
		game.load.tilemap('gamemap', 'tilemap.json', null, Phaser.Tilemap.TILED_JSON);

	}

	create() {

		game.world.setBounds(0, 0, 800, 600);
		game.stage.backgroundColor = '#000000';
		game.physics.startSystem(Phaser.Physics.ARCADE);
		console.log("Debug enabled:", !game.debug.isDisabled);

		var map = game.add.tilemap('gamemap');
		map.addTilesetImage('tileset', 'tileset');
		map.addTilesetImage('objects', 'objects');
		var layer_floor = map.createLayer('floor');
		layer_floor.resizeWorld();
		this.layer_walls = map.createLayer('walls');
		map.setCollisionBetween(1, 100, true, this.layer_walls);
		this.layer_furniture = map.createLayer('furniture');
		map.setCollisionBetween(1, 100, true, this.layer_furniture);

		// FIXME: Don't create a group here like this, it's too ugly! Now I have it here due to the display order.
		logic.doors = game.add.group(game.world, "doors");

		map.objects['objects'].forEach(function(o) { logic.createObject(o); });
		console.log(map.objects);

		cursors = game.input.keyboard.createCursorKeys();
		console.log(logic);

	}

	update() {

		game.physics.arcade.collide(logic.player, this.layer_walls);
		game.physics.arcade.collide(logic.player, this.layer_furniture);
		game.physics.arcade.collide(logic.player, logic.doors);
		logic.update();

	}
}
